# desidulate

## Overview

desidulate is a command line python suite of tools for understanding C64/SID music - helping understand how the SID is used, and translating SID music (with limitations, see below) to MIDI format. desidulate works directly on SID register logfiles (as generated by VICE's `-sounddev dump` option), so it can parse music from any C64 software (game, demo, .sid file, etc) and can leverage all the fine existing and ongoing work within VICE dealing with hardware quirks, illegal opcodes in play routines, etc.

desidulate is intended to help new SID composers understand the SID more quickly and completely, find unexplored territory, and build upon the SID legacy. desidulate was also written to assist [CHIME RED](https://github.com/anarkiwi/chime_red2) Tesla coil synthesizer research - how to apply SID style synthesis to high voltage audio generators, and to avoid error-prone audio based transcription (such has that built into Ableton Live). desidulate doesn't require extensive dependencies and is intended for command line use (e.g. to analyze 1000s of SID files efficiently), and is based on Pandas.

## Installing

```
$ pip3 install .
```

## Transcribing to a MIDI file

1. Generate VICE register dump (by default, generates vicesnd.sid in current directory).

```
$ x64sc -sounddev dump /path/to/game.d64
$ mv vicesnd.sid mytune.dump
```

or

```
$ vsid -sounddev dump /path/to/tune.sid
$ mv vicesnd.sid mytune.dump
```

2. Generate `ssf` and `log` files.

```
$ reg2ssf mytune.dump
```

This intermediate step, identifies all SSFs (see below) and writes them to `mytune.ssf.xz`, and also writes a log file `mytune.log.xz`, which logs when SSFs are played (in CPU clock cycles).

3. Generate SMF MIDI file.

```
$ ssf2midi mytune.log.xz
```

This re-renders the `ssf` and `log` files into a SMF `mid` file, `mytune.mid`

desidulate can generate a multitrack SMF (one track for each voice, and an additional track for each voice for percussion). The intent is not perfect MIDI reproduction (not possible due to missing features in MIDI like standardized support for filter sweeps, etc) but to allow analysis of SID programming techniques (e.g. how a particular kick sound is made), and to allow a composer to have MIDI based devices accompany a C64 composition without complex hardware integration.

A SID register dump is parsed into partitions called SSFs - SID Sound Fragments, which are divided on SID voice gate off to on events. The test bit is accounted for as well, to partition on "hard restart" type events. Then each partition is analyzed for the closest MIDI notes based on frequency, and what waveforms are used while a voice is gated on. More than one note can be generated if the pitch channges sufficiently while the gate is on. Notes then are either classed as "regular" (no use of noise waveform) or "percussion" (use of noise waveform). Percussion events are then partitioned further - events that use the noise waveform only are transcribed to hihat, etc, based on noise length. Snares are detected by use of the noise waveform repeatedly used before and after another waveform. Kick drums are detected by use of a pitch drop.

## Transcibing an SSF to a WAV file

```
$ ssf2wav mytune.ssf.xz <hashid>
```

Where `<hashid>` is the key of the SSF to render (view `mytune.ssf.xz` for the complete list of SSF's detected, sorted by most often used SSF first).


## Transcribing to a WAV file

```
$ reg2wav mytune.dump
```

The result will be rendered into `mytune.wav`.

desidulate can generate a WAV file from a ReSID based SID simulation directly from a VICE register dump. This allows the composer to write SID music by directly inputting SID registers (either manually or perhaps from a progressive algorithm) without having to deal with a C64 emulator or an intervening MIDI translation layer (like a SIDStation) that is convenient but also limits expression.

## Transcribing an SSF to a Sid Wizard instrument

desidulate can, with some limitations, transcribe an SSF to a Sid Wizard instrument.

Extract the SID register dump from the song using VICE:

```
vsid -sounddev dump -soundarg C64Music/MUSICIANS/L/Linus/Cauldron_II_Remix.sid.dump -warp -limit 300000000 C64Music/MUSICIANS/L/Linus/Cauldron_II_Remix.sid
```

Process the desired song into an SSF log:

```
reg2ssf C64Music/MUSICIANS/L/Linus/Cauldron_II_Remix.sid.dump
```

Render all the SSFs as WAVs:

```
ssf2wav C64Music/MUSICIANS/L/Linus/Cauldron_II_Remix.ssf.xz
```

Listen to the WAVs, to find the instrument you wish to transcribe, which will be indentified with a hashid in the filename.

Now, transcribe the SSF into an instrument (in this case, a kick sound): desidulate attempts to optimize the instruments by detecting and automating filter and PWM curves.

```
ssf2swi C64Music/MUSICIANS/L/Linus/Cauldron_II_Remix.ssf.xz -1975247557004053752
multispeed: 1
ADSR: 05F0

     F WFARP PULSE  FILT
0   00  81BB  ....  BF60
1   01  41A7  88..  BF03
2   02  419B  84..  02F8
3   03  ....  0340  BF12
4   04  ....  85..  BF06
5   05  ....  0340  BF02
6   06  ....  86..  BF03
7   07  ....  0240  ....
8   08  ....  ....  ....
9   09  ....  ....  ....
10  0A  ....  ....  ....
11  0B  ....  ....  ....
12  0C  ....  ....  ....

    gate1  freq1  pwduty1  pulse1  noise1  tri1  saw1  test1  sync1  ring1  freq3  test3  flt1  fltcoff  fltres  fltlo  fltband  flthi  real_freq  closest_note
0       1   7940     <NA>       0       1     0     0      0   <NA>   <NA>   <NA>   <NA>     1      768      15      1        1      0     466.28            70
1       1   2501     2048       1       0     0     0      0   <NA>   <NA>   <NA>   <NA>     1       24      15      1        1      0     146.87            50
2       1   1250     1024       1       0     0     0      0   <NA>   <NA>   <NA>   <NA>     1       16      15      1        1      0      73.41            38
3       1   1250     1088       1       0     0     0      0   <NA>   <NA>   <NA>   <NA>     1        8      15      1        1      0      73.41            38
4       1   1250     1152       1       0     0     0      0   <NA>   <NA>   <NA>   <NA>     1      144      15      1        1      0      73.41            38
5       1   1250     1216       1       0     0     0      0   <NA>   <NA>   <NA>   <NA>     1       48      15      1        1      0      73.41            38
6       1   1250     1280       1       0     0     0      0   <NA>   <NA>   <NA>   <NA>     1       16      15      1        1      0      73.41            38
7       1   1250     1344       1       0     0     0      0   <NA>   <NA>   <NA>   <NA>     1       24      15      1        1      0      73.41            38
8       1   1252     1408       1       0     0     0      0   <NA>   <NA>   <NA>   <NA>     1       24      15      1        1      0      73.52            38
9       1   1254     1472       1       0     0     0      0   <NA>   <NA>   <NA>   <NA>     1       24      15      1        1      0      73.64            38
10      1   1256     1536       1       0     0     0      0   <NA>   <NA>   <NA>   <NA>     1       24      15      1        1      0      73.76            38
11      1   1254     1600       1       0     0     0      0   <NA>   <NA>   <NA>   <NA>     1       24      15      1        1      0      73.64            38
12      1   1252     1664       1       0     0     0      0   <NA>   <NA>   <NA>   <NA>     1       24      15      1        1      0      73.52            38

```

Now enter the values into SID Wizard (a direct SWI exporter is planned), and play the sound with "z!"

![Alt text](docs/swishot.png?raw=true "swi screenshot")


## Ongoing work

* Pitchbending for following small pitch changes ([Magenta](https://github.com/magenta/magenta/issues/1902) does not use pitchbends as of March 2021)
* Export to [defMON](https://www.vandervecken.com/defmon)

## References

* https://github.com/M3wP/XSID: XSID, SID analyzer and MIDI converter (targets Windows/Visual Studio/Delphi primarily)
* https://haendel.ddns.net/~ken/: JSIDPlay2, C64 emulator with SID register logging (requires Java and is resource intensive on some platforms)
* https://csdb.dk/release/?id=152422: siddump, C64 SID analyzer (some limitations with illegal opcodes and intended for use with .sid files specifically)
